
CREATE TABLE changroup
(
  groupid int4,
  a1 int4,
  a2 int4,
  a3 int4,
  a4 int4
) 
WITHOUT OIDS;
ALTER TABLE changroup OWNER TO postgres;


CREATE TABLE groupname
(
  groupid int4,
  name text
) 
WITHOUT OIDS;
ALTER TABLE groupname OWNER TO postgres;


CREATE TYPE interval_parts AS
   (name text,
    "year" float8,
    "month" float8,
    "day" float8,
    "hour" float8,
    "minute" float8,
    "second" float8,
    time_interval interval);
ALTER TYPE interval_parts OWNER TO postgres;


CREATE TYPE getliftstay_result AS
   (name text,
    time_interval interval);
ALTER TYPE getliftstay_result OWNER TO postgres;


-- Function: getliftstay("timestamp", "timestamp", int4)
-- DROP FUNCTION getliftstay("timestamp", "timestamp", int4);
CREATE OR REPLACE FUNCTION getliftstay("timestamp", "timestamp", int4)
  RETURNS SETOF interval_parts AS
$BODY$

DECLARE
stime 	ALIAS FOR $1;
etime 	ALIAS FOR $2;
id		ALIAS FOR $3;
temprec 	  record;
chanrec		  record;
res			  interval;
full_interval interval;
goodtime	  "timestamp";
badtime	  	  "timestamp";
nulltime 	  "timestamp";
ret		  	  getliftstay_result;
ret_interval interval_parts;
BEGIN
	ret_interval.name='';
	ret_interval.year = 0;
	ret_interval.month = 0;
	ret_interval.day=0;
	ret_interval.hour=0;
	ret_interval.minute=0;
	ret_interval.second=0;
	
	res := interval'0 days';
	full_interval := interval'0 days';
	ret.name :='';
	ret.time_interval:=interval'0 days';
	nulltime:=timestamp'1970-01-01';
	badtime :=nulltime;
	goodtime:=nulltime;
	--   
	FOR chanrec IN SELECT c.name,cg.a1,cg.a2,cg.a3,cg.a4 FROM changroup cg
	LEFT JOIN channels c ON (cg.a1 = c.a1 AND cg.a2 = c.a2 AND cg.a3 = c.a3 AND cg.a4 = c.a4)
	WHERE cg.groupid = id AND c.dtype = 24  LOOP
	
	ret.name :=chanrec.name;
	ret_interval.name:=chanrec.name;
	res := interval'0 days';	
	-- 
	FOR temprec IN SELECT l.rtime,l.state,l.val FROM logs l 
		WHERE  l.a1 =chanrec.a1 AND l.a2 =chanrec.a2 AND l.a3 =chanrec.a3 AND l.a4 =chanrec.a4
		AND l.rtime <=stime AND l.dtype = 24
        ORDER BY rtime DESC LIMIT 1 LOOP
	END LOOP;
	IF (found) THEN
		IF (temprec.state = 0) THEN
			IF (getvalue(cast(24 as int2),temprec.val,cast(0 as int4)) IN (0,3,4,8,9,10,11,12,14,17,18,22,23,24,25) ) THEN
				goodtime := stime;
			ELSE
				badtime := stime;
			END IF;
		ELSE
			badtime := stime;
		END IF;
	ELSE
		badtime:=stime;
	END IF;
	--     
	FOR temprec IN SELECT l.rtime,l.state,l.val FROM logs l 
		WHERE  l.a1 =chanrec.a1 AND l.a2 =chanrec.a2 AND l.a3 =chanrec.a3 AND l.a4 =chanrec.a4 
		AND l.rtime >stime AND l.rtime <= etime AND l.dtype = 24 ORDER BY rtime ASC LOOP
		IF (temprec.state = 0) THEN
			--   
			IF (getvalue(cast(24 as int2),temprec.val,cast(0 as int4)) IN (0,3,4,8,9,10,11,12,14,17,18,22,23,24,25) ) THEN
				IF (badtime <> nulltime) THEN		
					res:=res+ (temprec.rtime - badtime);
					badtime := nulltime;
				END IF;
				goodtime := temprec.rtime;
			ELSE
				IF (badtime = nulltime) THEN			
					badtime := temprec.rtime;
				END IF;
			END IF;
		ELSE
			IF (badtime = nulltime) THEN
				badtime := temprec.rtime;
			END IF;
		END IF;	
	END LOOP;
	IF (found) THEN
		--  
		--   - ,   -    
		IF (temprec.state = 0) THEN
			IF (getvalue(cast(24 as int2),temprec.val,cast(0 as int4)) NOT IN (0,3,4,8,9,10,11,12,14,17,18,22,23,24,25)) THEN
				res:=res + (etime - temprec.rtime);
			END IF;
		ELSE
			res:=res + (etime - temprec.rtime);
		END IF;
	ELSE 
		IF (badtime <> nulltime) THEN
			res:=res+(etime - stime);
		END IF;
	END IF;
	res:=justify_hours(res);
	res:=justify_days(res);
	ret.time_interval:=res;
	full_interval := full_interval + res;
	
	ret_interval.year 	:= 	EXTRACT(year from res);
	ret_interval.month 	:= 	EXTRACT(month from res);
	ret_interval.day	:=	EXTRACT(day from res);
	ret_interval.hour	:=	EXTRACT(hour from res);
	ret_interval.minute	:=	EXTRACT(minute from res);
	ret_interval.second	:=	EXTRACT(second from res);
	ret_interval.time_interval:=res;
	ret_interval.month  := ret_interval.month + ret_interval.year*12;
	ret_interval.year:=0;
	RETURN NEXT ret_interval;
END LOOP;
ret.name :='';
full_interval:=justify_hours(full_interval);
full_interval:=justify_days(full_interval);
ret.time_interval := full_interval;
	ret_interval.name:='';
	ret_interval.year 	:= 	EXTRACT(year from full_interval);
	ret_interval.month 	:= 	EXTRACT(month from full_interval);
	ret_interval.day	:=	EXTRACT(day from full_interval);
	ret_interval.hour	:=	EXTRACT(hour from full_interval);
	ret_interval.minute	:=	EXTRACT(minute from full_interval);
	ret_interval.second	:=	EXTRACT(second from full_interval);
	ret_interval.time_interval:=full_interval;
	ret_interval.month  := ret_interval.month + ret_interval.year*12;
	ret_interval.year:=0;
RETURN NEXT ret_interval;
RETURN;
END;
$BODY$
  LANGUAGE 'plpgsql' IMMUTABLE STRICT;
ALTER FUNCTION getliftstay("timestamp", "timestamp", int4) OWNER TO postgres;


CREATE OR REPLACE FUNCTION getbroketime("timestamp", "timestamp", int4)
  RETURNS SETOF interval_parts AS
$BODY$

DECLARE
stime 	ALIAS FOR $1;
etime 	ALIAS FOR $2;
id		ALIAS FOR $3;-- 
temprec 	  record;
chanrec		  record;
res			  interval;
full_interval interval;
goodtime	  "timestamp";
badtime	  	  "timestamp";
nulltime 	  "timestamp";
ret		  	  getliftstay_result;
ret_interval interval_parts;
BEGIN
	ret_interval.name='';
	ret_interval.year = 0;
	ret_interval.month = 0;
	ret_interval.day=0;
	ret_interval.hour=0;
	ret_interval.minute=0;
	ret_interval.second=0;
	
	res := interval'0 days';
	full_interval := interval'0 days';
	ret.name :='';
	ret.time_interval:=interval'0 days';
	nulltime:=timestamp'1970-01-01';
	badtime :=nulltime;
	goodtime:=nulltime;
	--   
	FOR chanrec IN SELECT c.name,cg.a1,cg.a2,cg.a3,cg.a4 FROM changroup cg
	LEFT JOIN channels c ON (cg.a1 = c.a1 AND cg.a2 = c.a2 AND cg.a3 = c.a3 AND cg.a4 = c.a4)
	WHERE cg.groupid = id  LOOP
	
	ret.name :=chanrec.name;
	ret_interval.name:=chanrec.name;
	res := interval'0 days';	
	-- 
	FOR temprec IN SELECT l.rtime,l.state,l.val FROM logs l 
		WHERE  l.a1 =chanrec.a1 AND l.a2 =chanrec.a2 AND l.a3 =chanrec.a3 AND l.a4 =chanrec.a4
		AND l.rtime <=stime
        ORDER BY rtime DESC LIMIT 1 LOOP
	END LOOP;
	IF (found) THEN
		IF (temprec.state IN (0,1,2,3)) THEN
				goodtime := stime;
		ELSE
			badtime := stime;
		END IF;
	ELSE
		badtime:=stime;
	END IF;
	--     
	FOR temprec IN SELECT l.rtime,l.state,l.val FROM logs l 
		WHERE  l.a1 =chanrec.a1 AND l.a2 =chanrec.a2 AND l.a3 =chanrec.a3 AND l.a4 =chanrec.a4 
		AND l.rtime >stime AND l.rtime <= etime  ORDER BY rtime ASC LOOP
		IF (temprec.state IN (0,1,2,3)) THEN
			--   
				IF (badtime <> nulltime) THEN		
					res:=res+ (temprec.rtime - badtime);
					badtime := nulltime;
				END IF;
				goodtime := temprec.rtime;
		ELSE
			IF (badtime = nulltime) THEN
				badtime := temprec.rtime;
			END IF;
		END IF;	
	END LOOP;
	IF (found) THEN
		--  
		--   - ,   -    
		IF (temprec.state IN (0,1,2,3)) THEN
				res:=res + (etime - temprec.rtime);
		ELSE
			res:=res + (etime - temprec.rtime);
		END IF;
	ELSE 
		IF (badtime <> nulltime) THEN
			res:=res+(etime - stime);
		END IF;
	END IF;
	res:=justify_hours(res);
	res:=justify_days(res);
	ret.time_interval:=res;
	full_interval := full_interval + res;
	
	ret_interval.year 	:= 	EXTRACT(year from res);
	ret_interval.month 	:= 	EXTRACT(month from res);
	ret_interval.day	:=	EXTRACT(day from res);
	ret_interval.hour	:=	EXTRACT(hour from res);
	ret_interval.minute	:=	EXTRACT(minute from res);
	ret_interval.second	:=	EXTRACT(second from res);
	ret_interval.time_interval:=res;
	ret_interval.month  := ret_interval.month + ret_interval.year*12;
	ret_interval.year:=0;
	RETURN NEXT ret_interval;
END LOOP;
ret.name :='';
full_interval:=justify_hours(full_interval);
full_interval:=justify_days(full_interval);
ret.time_interval := full_interval;
	ret_interval.name:='';
	ret_interval.year 	:= 	EXTRACT(year from full_interval);
	ret_interval.month 	:= 	EXTRACT(month from full_interval);
	ret_interval.day	:=	EXTRACT(day from full_interval);
	ret_interval.hour	:=	EXTRACT(hour from full_interval);
	ret_interval.minute	:=	EXTRACT(minute from full_interval);
	ret_interval.second	:=	EXTRACT(second from full_interval);
	ret_interval.time_interval:=full_interval;
	ret_interval.month  := ret_interval.month + ret_interval.year*12;
	ret_interval.year:=0;
RETURN NEXT ret_interval;
RETURN;
END;
$BODY$
  LANGUAGE 'plpgsql' IMMUTABLE STRICT;
ALTER FUNCTION getbroketime("timestamp", "timestamp", int4) OWNER TO postgres;


CREATE TYPE get_group_values_names_result AS
   (name text,
    rtime timestamp,
    statename text,
    state int2);
ALTER TYPE get_group_values_names_result OWNER TO postgres;

CREATE OR REPLACE FUNCTION get_group_values("timestamp", "timestamp", int4)
  RETURNS SETOF get_group_values_names_result AS
$BODY$

DECLARE
stime 	ALIAS FOR $1;
etime 	ALIAS FOR $2;
id		ALIAS FOR $3;
res get_group_values_names_result;
temprec 	  record;
chanrec		  record;
BEGIN
	--   
	FOR chanrec IN SELECT c.name,cg.a1,cg.a2,cg.a3,cg.a4 FROM changroup cg
	LEFT JOIN channels c ON (cg.a1 = c.a1 AND cg.a2 = c.a2 AND cg.a3 = c.a3 AND cg.a4 = c.a4)
	WHERE cg.groupid = id LOOP
	FOR temprec IN SELECT values.rtime, values.val, values.state, values.dtype FROM get_values(chanrec.a1,chanrec.a2,chanrec.a3,chanrec.a4,stime,etime ) AS values LOOP
		IF temprec.state <> 0 THEN
			FOR res IN SELECT chanrec.name,temprec.rtime,state.name, temprec.state FROM state WHERE state.stateid = temprec.state LOOP
					RETURN NEXT res;
			END LOOP;
		ELSE
			IF temprec.dtype IN (11,12,13,14,15,16,17,18,19,20,21,24,25,254,255) THEN
				FOR res IN SELECT chanrec.name,temprec.rtime,valuename.val_name,temprec.state FROM valuename WHERE valuename.dtype = temprec.dtype AND valuename.val = temprec.val LOOP
					RETURN NEXT res;
				END LOOP;
			ELSE
				FOR res IN SELECT  chanrec.name,temprec.rtime,round(cast(temprec.val as numeric),2), temprec.state LOOP
					RETURN NEXT res;
				END LOOP;
			END IF;
		END IF;
	END LOOP;
	END LOOP;
RETURN;
END;
$BODY$
  LANGUAGE 'plpgsql' IMMUTABLE STRICT;



ALTER FUNCTION get_group_values("timestamp", "timestamp", int4) OWNER TO postgres;
CREATE OR REPLACE FUNCTION get_group_values_last("timestamp", int4)
  RETURNS SETOF get_group_values_names_result AS
$BODY$

DECLARE
etime 	ALIAS FOR $1;
id		ALIAS FOR $2;
res get_group_values_names_result;
temprec 	  record;
chanrec		  record;
BEGIN
	--   
	FOR chanrec IN SELECT c.name,cg.a1,cg.a2,cg.a3,cg.a4 FROM changroup cg
	LEFT JOIN channels c ON (cg.a1 = c.a1 AND cg.a2 = c.a2 AND cg.a3 = c.a3 AND cg.a4 = c.a4)
	WHERE cg.groupid = id LOOP
	FOR temprec IN SELECT rtime, getvalue(dtype,val,0),dtype, state FROM logs WHERE a1=chanrec.a1 AND a2=chanrec.a2 AND a3=chanrec.a3 AND a4=chanrec.a4 AND rtime <= etime ORDER BY rtime DESC LIMIT 1 LOOP
	IF temprec.state <> 0 THEN
			FOR res IN SELECT chanrec.name,temprec.rtime,state.name, temprec.state FROM state WHERE state.stateid = temprec.state LOOP
					RETURN NEXT res;
			END LOOP;
		ELSE
			IF temprec.dtype IN (11,12,13,14,15,16,17,18,19,20,21,24,25,254,255) THEN
				FOR res IN SELECT chanrec.name,temprec.rtime,valuename.val_name,temprec.state FROM valuename WHERE valuename.dtype = temprec.dtype AND valuename.val = temprec.val LOOP
					RETURN NEXT res;
				END LOOP;
			ELSE
				FOR res IN SELECT  chanrec.name,temprec.rtime,round(cast(temprec.val as numeric),2), temprec.state LOOP
					RETURN NEXT res;
				END LOOP;
			END IF;
		END IF;	
	END LOOP;
	END LOOP;
RETURN;
END;
$BODY$
  LANGUAGE 'plpgsql' IMMUTABLE STRICT;
ALTER FUNCTION get_group_values_last( "timestamp", int4) OWNER TO postgres;
